@@ -11,7 +11,7 @@ use core::mem;
 use core::ptr::NonNull;
 use core::slice::Iter as SliceIter;
 
-use crate::alloc::boxed::Box;
+use crate::alloc::{boxed::Box, vec::Vec};
 use crate::archetype::Archetype;
 use crate::entities::EntityMeta;
 use crate::{Component, Entity, World};
@@ -22,12 +22,17 @@ pub trait Query {
     type Fetch: for<'a> Fetch<'a>;
 }
 
+/// Marker trait indicating whether a given [`Query`] will not produce unique references
+#[allow(clippy::missing_safety_doc)]
+pub unsafe trait QueryShared {}
+
 /// Type of values yielded by a query
 ///
 /// Once rust offers generic associated types, this will be moved into [`Query`].
 pub type QueryItem<'a, Q> = <<Q as Query>::Fetch as Fetch<'a>>::Item;
 
 /// Streaming iterators over contiguous homogeneous ranges of components
+#[allow(clippy::missing_safety_doc)]
 pub unsafe trait Fetch<'a>: Sized {
     /// Type of value to be fetched
     type Item;
@@ -79,6 +84,8 @@ impl<'a, T: Component> Query for &'a T {
     type Fetch = FetchRead<T>;
 }
 
+unsafe impl<'a, T> QueryShared for &'a T {}
+
 #[doc(hidden)]
 pub struct FetchRead<T>(NonNull<T>);
 
@@ -172,6 +179,8 @@ impl<T: Query> Query for Option<T> {
     type Fetch = TryFetch<T::Fetch>;
 }
 
+unsafe impl<T: QueryShared> QueryShared for Option<T> {}
+
 #[doc(hidden)]
 pub struct TryFetch<T>(Option<T>);
 
@@ -310,6 +319,8 @@ impl<L: Query, R: Query> Query for Or<L, R> {
     type Fetch = FetchOr<L::Fetch, R::Fetch>;
 }
 
+unsafe impl<L: QueryShared, R: QueryShared> QueryShared for Or<L, R> {}
+
 #[doc(hidden)]
 pub struct FetchOr<L, R>(Or<L, R>);
 
@@ -375,6 +386,8 @@ impl<T: Component, Q: Query> Query for Without<T, Q> {
     type Fetch = FetchWithout<T, Q::Fetch>;
 }
 
+unsafe impl<T, Q: QueryShared> QueryShared for Without<T, Q> {}
+
 #[doc(hidden)]
 pub struct FetchWithout<T, F>(F, PhantomData<fn(T)>);
 
@@ -445,6 +458,8 @@ impl<T: Component, Q: Query> Query for With<T, Q> {
     type Fetch = FetchWith<T, Q::Fetch>;
 }
 
+unsafe impl<T, Q: QueryShared> QueryShared for With<T, Q> {}
+
 #[doc(hidden)]
 pub struct FetchWith<T, F>(F, PhantomData<fn(T)>);
 
@@ -516,6 +531,8 @@ impl<Q: Query> Query for Satisfies<Q> {
     type Fetch = FetchSatisfies<Q::Fetch>;
 }
 
+unsafe impl<Q> QueryShared for Satisfies<Q> {}
+
 #[doc(hidden)]
 pub struct FetchSatisfies<F>(bool, PhantomData<F>);
 
@@ -575,6 +592,12 @@ impl<'w, Q: Query> QueryBorrow<'w, Q> {
         unsafe { QueryIter::new(self.meta, self.archetypes.iter()) }
     }
 
+    /// Provide random access to the query results
+    pub fn view(&mut self) -> View<'_, Q> {
+        self.borrow();
+        unsafe { View::new(self.meta, self.archetypes) }
+    }
+
     /// Like `iter`, but returns child iterators of at most `batch_size` elements
     ///
     /// Useful for distributing work over a threadpool.
@@ -770,6 +793,11 @@ impl<'q, Q: Query> QueryMut<'q, Q> {
         }
     }
 
+    /// Provide random access to the query results
+    pub fn view(&mut self) -> View<'_, Q> {
+        unsafe { View::new(self.iter.meta, self.iter.archetypes.as_slice()) }
+    }
+
     /// Transform the query into one that requires a certain component without borrowing it
     ///
     /// See `QueryBorrow::with`
@@ -923,7 +951,7 @@ impl<'q, Q: Query> Iterator for BatchedIter<'q, Q> {
     }
 }
 
-/// A sequence of entities yielded by `BatchedIter`
+/// A sequence of entities yielded by [`BatchedIter`]
 pub struct Batch<'q, Q: Query> {
     meta: &'q [EntityMeta],
     state: ChunkIter<Q>,
@@ -1004,6 +1032,8 @@ macro_rules! tuple_impl {
         impl<$($name: Query),*> Query for ($($name,)*) {
             type Fetch = ($($name::Fetch,)*);
         }
+
+        unsafe impl<$($name: QueryShared),*> QueryShared for ($($name,)*) {}
     };
 }
 
@@ -1014,6 +1044,7 @@ smaller_tuples_too!(tuple_impl, O, N, M, L, K, J, I, H, G, F, E, D, C, B, A);
 pub struct PreparedQuery<Q: Query> {
     memo: (u64, u64),
     state: Box<[(usize, <Q::Fetch as Fetch<'static>>::State)]>,
+    fetch: Box<[Option<Q::Fetch>]>,
 }
 
 impl<Q: Query> Default for PreparedQuery<Q> {
@@ -1026,9 +1057,10 @@ impl<Q: Query> PreparedQuery<Q> {
     /// Create a prepared query which is not yet attached to any world
     pub fn new() -> Self {
         Self {
-            // This memo will not match any world as the first ID will 1.
+            // This memo will not match any world as the first ID will be 1.
             memo: (0, 0),
             state: Default::default(),
+            fetch: Default::default(),
         }
     }
 
@@ -1042,7 +1074,9 @@ impl<Q: Query> PreparedQuery<Q> {
             .filter_map(|(idx, x)| Q::Fetch::prepare(x).map(|state| (idx, state)))
             .collect();
 
-        Self { memo, state }
+        let fetch = world.archetypes().map(|_| None).collect();
+
+        Self { memo, state, fetch }
     }
 
     /// Query `world`, using dynamic borrow checking
@@ -1057,7 +1091,7 @@ impl<Q: Query> PreparedQuery<Q> {
         let meta = world.entities_meta();
         let archetypes = world.archetypes_inner();
 
-        PreparedQueryBorrow::new(meta, archetypes, &*self.state)
+        PreparedQueryBorrow::new(meta, archetypes, &*self.state, &mut *self.fetch)
     }
 
     /// Query a uniquely borrowed world
@@ -1078,6 +1112,23 @@ impl<Q: Query> PreparedQuery<Q> {
 
         unsafe { PreparedQueryIter::new(meta, archetypes, state.iter()) }
     }
+
+    /// Provide random access to query results for a uniquely borrow world
+    pub fn view_mut<'q>(&'q mut self, world: &'q mut World) -> PreparedView<'q, Q> {
+        assert_borrow::<Q>();
+
+        if self.memo != world.memo() {
+            *self = Self::prepare(world);
+        }
+
+        let meta = world.entities_meta();
+        let archetypes = world.archetypes_inner();
+
+        let state: &'q [(usize, <Q::Fetch as Fetch<'q>>::State)] =
+            unsafe { mem::transmute(&*self.state) };
+
+        unsafe { PreparedView::new(meta, archetypes, state.iter(), &mut *self.fetch) }
+    }
 }
 
 /// Combined borrow of a [`PreparedQuery`] and a [`World`]
@@ -1085,6 +1136,7 @@ pub struct PreparedQueryBorrow<'q, Q: Query> {
     meta: &'q [EntityMeta],
     archetypes: &'q [Archetype],
     state: &'q [(usize, <Q::Fetch as Fetch<'static>>::State)],
+    fetch: &'q mut [Option<Q::Fetch>],
 }
 
 impl<'q, Q: Query> PreparedQueryBorrow<'q, Q> {
@@ -1092,6 +1144,7 @@ impl<'q, Q: Query> PreparedQueryBorrow<'q, Q> {
         meta: &'q [EntityMeta],
         archetypes: &'q [Archetype],
         state: &'q [(usize, <Q::Fetch as Fetch<'static>>::State)],
+        fetch: &'q mut [Option<Q::Fetch>],
     ) -> Self {
         for (idx, state) in state {
             Q::Fetch::borrow(&archetypes[*idx], *state);
@@ -1101,6 +1154,7 @@ impl<'q, Q: Query> PreparedQueryBorrow<'q, Q> {
             meta,
             archetypes,
             state,
+            fetch,
         }
     }
 
@@ -1112,6 +1166,14 @@ impl<'q, Q: Query> PreparedQueryBorrow<'q, Q> {
 
         unsafe { PreparedQueryIter::new(self.meta, self.archetypes, state.iter()) }
     }
+
+    /// Provides random access to the results of the prepared query
+    pub fn view<'i>(&'i mut self) -> PreparedView<'i, Q> {
+        let state: &'i [(usize, <Q::Fetch as Fetch<'i>>::State)] =
+            unsafe { mem::transmute(self.state) };
+
+        unsafe { PreparedView::new(self.meta, self.archetypes, state.iter(), self.fetch) }
+    }
 }
 
 impl<Q: Query> Drop for PreparedQueryBorrow<'_, Q> {
@@ -1199,6 +1261,123 @@ impl<Q: Query> ExactSizeIterator for PreparedQueryIter<'_, Q> {
     }
 }
 
+/// Provides random access to the results of a query
+pub struct View<'q, Q: Query> {
+    meta: &'q [EntityMeta],
+    fetch: Vec<Option<Q::Fetch>>,
+}
+
+impl<'q, Q: Query> View<'q, Q> {
+    /// # Safety
+    ///
+    /// `'q` must be sufficient to guarantee that `Q` cannot violate borrow safety, either with
+    /// dynamic borrow checks or by representing exclusive access to the `World`.
+    unsafe fn new(meta: &'q [EntityMeta], archetypes: &'q [Archetype]) -> Self {
+        let fetch = archetypes
+            .iter()
+            .map(|archetype| {
+                Q::Fetch::prepare(archetype).map(|state| Q::Fetch::execute(archetype, state))
+            })
+            .collect();
+
+        Self { meta, fetch }
+    }
+
+    /// Retrieve the query results corresponding to `entity`
+    ///
+    /// Will yield `None` if the entity does not exist or does not match the query.
+    ///
+    /// Does not require exclusive access to the map, but is defined only for queries yielding only shared references.
+    pub fn get<'m>(&'m self, entity: Entity) -> Option<QueryItem<'q, Q>>
+    where
+        Q: QueryShared,
+    {
+        let meta = self.meta.get(entity.id as usize)?;
+        if meta.generation != entity.generation {
+            return None;
+        }
+
+        self.fetch[meta.location.archetype as usize]
+            .as_ref()
+            .map(|fetch| unsafe { fetch.get(meta.location.index as usize) })
+    }
+
+    /// Retrieve the query results corresponding to `entity`
+    ///
+    /// Will yield `None` if the entity does not exist or does not match the query.
+    pub fn get_mut<'m>(&'m mut self, entity: Entity) -> Option<QueryItem<'q, Q>> {
+        let meta = self.meta.get(entity.id as usize)?;
+        if meta.generation != entity.generation {
+            return None;
+        }
+
+        self.fetch[meta.location.archetype as usize]
+            .as_ref()
+            .map(|fetch| unsafe { fetch.get(meta.location.index as usize) })
+    }
+}
+
+/// Provides random access to the results of a prepared query
+pub struct PreparedView<'q, Q: Query> {
+    meta: &'q [EntityMeta],
+    fetch: &'q mut [Option<Q::Fetch>],
+}
+
+impl<'q, Q: Query> PreparedView<'q, Q> {
+    /// # Safety
+    ///
+    /// `'q` must be sufficient to guarantee that `Q` cannot violate borrow safety, either with
+    /// dynamic borrow checks or by representing exclusive access to the `World`.
+    unsafe fn new(
+        meta: &'q [EntityMeta],
+        archetypes: &'q [Archetype],
+        state: SliceIter<'q, (usize, <Q::Fetch as Fetch<'q>>::State)>,
+        fetch: &'q mut [Option<Q::Fetch>],
+    ) -> Self {
+        fetch.iter_mut().for_each(|fetch| *fetch = None);
+
+        for (idx, state) in state {
+            let archetype = &archetypes[*idx];
+            fetch[*idx] = Some(Q::Fetch::execute(archetype, *state));
+        }
+
+        Self { meta, fetch }
+    }
+
+    /// Retrieve the query results corresponding to `entity`
+    ///
+    /// Will yield `None` if the entity does not exist or does not match the query.
+    ///
+    /// Does not require exclusive access to the map, but is defined only for queries yielding only shared references.
+    pub fn get<'m>(&'m self, entity: Entity) -> Option<QueryItem<'q, Q>>
+    where
+        Q: QueryShared,
+    {
+        let meta = self.meta.get(entity.id as usize)?;
+        if meta.generation != entity.generation {
+            return None;
+        }
+
+        self.fetch[meta.location.archetype as usize]
+            .as_ref()
+            .map(|fetch| unsafe { fetch.get(meta.location.index as usize) })
+    }
+
+    /// Retrieve the query results corresponding to `entity`
+    ///
+    /// Will yield `None` if the entity does not exist or does not match the query.
+    pub fn get_mut<'m>(&'m mut self, entity: Entity) -> Option<QueryItem<'q, Q>> {
+        let meta = self.meta.get(entity.id as usize)?;
+        if meta.generation != entity.generation {
+            return None;
+        }
+
+        self.fetch[meta.location.archetype as usize]
+            .as_ref()
+            .map(|fetch| unsafe { fetch.get(meta.location.index as usize) })
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
